# -*- coding: utf-8 -*- 
# @Time    : 03/21/2024 
# @Author  : nzooherd 
# @File    : payment.py 
# @Software: Visual Studio Code


from ast import Call
from logging import getLogger
from telethon.events import filters, NewMessage, ButtonCallback
from telethon.types import Message

from aspect.api import ApiAspect
from controller import BaseController, CallbackPrefix, RedeemMatch
from exception.insufficient_balance import InsufficientBalanceException
from exception.message_notfound import MessageNotFoundException
from service.message_service import MessageService
from service.payment_service import PaymentService
from service.video_service import VideoService
from utills.bytes_util import CallbackBytesRequest

logger = getLogger()

class PaymentController(BaseController):

    def __init__(self, mybot) -> None:
        super().__init__(mybot)
        
        self.payment_service = PaymentService()
        self.message_service = MessageService()
        self.video_service = VideoService()


    @ApiAspect(filter=filters.Any(filters.Command("/redeem"), RedeemMatch()))
    async def redeem(self, update: NewMessage) -> Message:
        if " " in update.text:
            gift_card = update.text.split(" ")[1] 
        else:
            gift_card = update.text
        redeem_template = await self.payment_service.active_gift_card(gift_card)
        return await self.mybot.bot_client.send_message(update.chat, template=redeem_template)

    @ApiAspect(filter=filters.Command("/account"))
    async def account(self, update: NewMessage) -> Message:
        account_template = await self.payment_service.account_info()
        return await self.mybot.bot_client.send_message(update.chat, template=account_template)

    @ApiAspect(filter=filters.Command("/purchase"))
    async def purchase(self, update: NewMessage) -> Message:
        video_md5 = int(update.text.split(" ")[1])
        await self.payment_service.purchase_video(video_md5)
        self.mybot.bot_client.edit_message(update.chat, update.message, text="Success!")
        return await self.mybot.bot_client.send_message(update.chat, text="Success!")
    

    @ApiAspect(filter=CallbackPrefix("/pc_c"))
    async def purchase_callback(self, update: ButtonCallback) -> Message:
        request: CallbackBytesRequest = CallbackBytesRequest.from_byte(["video_md5"], [32], update.data)
        video_md5 = request.video_md5.decode("utf-8")
        message_id = request.message_id
        chat = list(update._chat_map.values())[0]
        try:
            await self.payment_service.purchase_video(video_md5)
            template = await self.video_service.get_video_message(video_md5)
            try:
                old_message = await self.message_service.select_by_id(message_id)
                await self.mybot.bot_client.edit_message(chat, old_message=old_message, template=template, link_preview=True)
                return await update.answer(text="Success")
            except MessageNotFoundException as e:
                logger.error(e)
                await self.mybot.bot_client.send_message(chat, template=template, link_preview=True)
        except InsufficientBalanceException:
            template = await self.payment_service.purchase_giftcard_prompt(video_md5)
            return await self.mybot.bot_client.send_message(chat, template=template, link_preview=True)
    